<?php

namespace Jlr\Tags;

use Jlr\Tags\Api\Client as BaseClient;
use Jlr\Tags\Api\Model\Tag;
use Jlr\Tags\Api\Model\TagGroup;
use Jlr\Tags\Contracts\Client as ClientContract;
use Jlr\Tags\Endpoints\CreateTag;
use Jlr\Tags\Endpoints\FetchTagGroups;
use Jlr\Tags\Endpoints\FetchTags;
use Jlr\Tags\Endpoints\UpdateTag;
use Jlr\Tags\Endpoints\UpdateTagGroup;

class Client extends BaseClient implements ClientContract
{
    /**
     * @return Api\Model\TagsGetResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function fetchTags(array $queryParameters = [], string $fetch = BaseClient::FETCH_OBJECT)
    {
        return $this->executePsr7Endpoint(new FetchTags($queryParameters), $fetch);
    }

    /**
     * @return Api\Model\TagsSlugPatchResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function updateTag(string $slug, Tag $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        return $this->executePsr7Endpoint(new UpdateTag($slug, $requestBody), $fetch);
    }

    /**
     * @return Api\Model\TagGroupsGetResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function fetchTagGroups(array $queryParameters = [], string $fetch = BaseClient::FETCH_OBJECT)
    {
        return $this->executePsr7Endpoint(new FetchTagGroups($queryParameters), $fetch);
    }

    /**
     * @return \Jlr\Tags\Api\Model\TagGroup|\Psr\Http\Message\ResponseInterface|null
     */
    public function fetchTagGroup(TagGroup $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        $response = $this->fetchTagGroups(['filter[id]' => $requestBody->getId()], $fetch);

        if ($fetch === BaseClient::FETCH_OBJECT) {
            return array_last($response->getData());
        }

        return $response;
    }

    /**
     * @return Api\Model\TagGroupsIdPatchResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function updateTagGroup(int $tagGroupId, TagGroup $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        return $this->executePsr7Endpoint(new UpdateTagGroup($tagGroupId, $requestBody), $fetch);
    }

    /**
     * @return Api\Model\TagGroupsIdPatchResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function createTag(Tag $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        return $this->executePsr7Endpoint(new CreateTag($requestBody), $fetch);
    }

    /**
     * @return Api\Model\TagGroupsIdPatchResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function fetchTag(Tag $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        $response = $this->fetchTags(['filter[id]' => $requestBody->getId()], $fetch);

        if ($fetch === BaseClient::FETCH_OBJECT) {
            return array_last($response->getData());
        }

        return $response;
    }

    public function equals(Tag $first, Tag $second): bool
    {
        return $first->getName() === $second->getName()
            && $first->getId() === $second->getId()
            && $first->getGroupId() === $second->getGroupId()
            && $first->getChildSlugs() == $second->getChildSlugs(); // phpcs:ignore
    }

    public static function fake(): FakeClient
    {
        return tap(new FakeClient, function (FakeClient $client) {
            app()->instance(ClientContract::class, $client);
        });
    }
}
