<?php

namespace Jlr\Tags;

use Illuminate\Support\Arr;
use Jlr\Tags\Api\Client as BaseClient;
use Jlr\Tags\Api\Model\Tag;
use Jlr\Tags\Api\Model\TagGroup;
use Jlr\Tags\Api\Model\TagGroupsGetResponse200;
use Jlr\Tags\Api\Model\TagGroupsIdPatchResponse200;
use Jlr\Tags\Api\Model\TagsGetResponse200;
use Jlr\Tags\Api\Model\TagsPostResponse200;
use Jlr\Tags\Api\Model\TagsSlugPatchResponse200;
use PHPUnit\Framework\Assert;

class FakeClient extends Client
{
    public function __construct()
    {
    }

    /**
     * @var array
     */
    protected $tagRequests = [];

    /**
     * @var array
     */
    protected $tagGroupRequests = [];

    /**
     * @var array
     */
    protected $createdTags = [];

    /**
     * @var array
     */
    protected $updatedTags = [];

    /**
     * @var array
     */
    protected $updatedTagGroups = [];

    /**
     * @var \Throwable
     */
    protected $shouldThrow;

    /**
     * @return Api\Model\TagGroupsIdPatchResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function fetchTag(Tag $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        return Arr::first(array_collapse($this->tagRequests), function ($tag, $key) use ($requestBody) {
            if ($key instanceof Tag) {
                $tag = $key;
            }

            return $tag->getId() === $requestBody->getId();
        });
    }

    /**
     * @return \Jlr\Tags\Api\Model\TagsGetResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function fetchTags(array $queryParameters = [], string $fetch = BaseClient::FETCH_OBJECT)
    {
        $this->checkIfShouldThrow();

        return (new TagsGetResponse200)->setData(array_shift($this->tagRequests));
    }

    /**
     * @ \Jlr\Tags\Api\Model\TagsPostResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function createTag(Tag $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        $this->checkIfShouldThrow();

        $responseTag = clone $requestBody;
        $this->createdTags[] = $responseTag;
        $responseTag->setId(str_slug($responseTag->getName()));

        return (new TagsPostResponse200)->setData($responseTag);
    }

    /**
     * @return \Jlr\Tags\Api\Model\TagsSlugPatchResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function updateTag(string $slug, Tag $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        $this->checkIfShouldThrow();

        $this->updatedTags[$slug] = $requestBody;

        return (new TagsSlugPatchResponse200)->setData($requestBody);
    }

    /**
     * @return \Jlr\Tags\Api\Model\TagGroupsGetResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function fetchTagGroups(array $queryParameters = [], string $fetch = BaseClient::FETCH_OBJECT)
    {
        $this->checkIfShouldThrow();

        return (new TagGroupsGetResponse200)->setData(array_shift($this->tagGroupRequests));
    }

    /**
     * @return \Jlr\Tags\Api\Model\TagGroup|\Psr\Http\Message\ResponseInterface|null
     */
    public function fetchTagGroup(TagGroup $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        return Arr::first(array_collapse($this->tagGroupRequests), function ($tag, $key) use ($requestBody) {
            if ($key instanceof TagGroup) {
                $tag = $key;
            }

            return $tag->getId() === $requestBody->getId();
        });
    }

    /**
     * @return \Jlr\Tags\Api\Model\TagGroupsIdPatchResponse200|\Psr\Http\Message\ResponseInterface|null
     */
    public function updateTagGroup(int $tagGroupId, TagGroup $requestBody, string $fetch = BaseClient::FETCH_OBJECT)
    {
        $this->checkIfShouldThrow();

        $this->updatedTagGroups[$tagGroupId] = $requestBody;

        $responseGroup = clone $requestBody;
        $responseGroup->setId($tagGroupId);

        return (new TagGroupsIdPatchResponse200)->setData($responseGroup);
    }

    public function setTags(array $tags): self
    {
        $this->tagRequests[] = $tags;

        return $this;
    }

    public function setTagGroups(array $tagGroups): self
    {
        $this->tagGroupRequests[] = $tagGroups;

        return $this;
    }

    public function getCreatedTags(): array
    {
        return $this->createdTags;
    }

    public function getUpdatedTags(): array
    {
        return $this->updatedTags;
    }

    public function getUpdatedTagGroups(): array
    {
        return $this->updatedTagGroups;
    }

    public function shouldThrow(\Throwable $throwable): self
    {
        $this->shouldThrow = $throwable;

        return $this;
    }

    protected function checkIfShouldThrow(): void
    {
        if ($this->shouldThrow) {
            throw $this->shouldThrow;
        }
    }

    public function assertTagUpdated(string $tagSlug): self
    {
        Assert::assertArrayHasKey($tagSlug, $this->getUpdatedTags(), 'Did not see tag ' . $tagSlug . ' updated');

        return $this;
    }

    public function assertTagGroupUpdated(int $tagGroupId): self
    {
        Assert::assertArrayHasKey(
            $tagGroupId,
            $this->getUpdatedTagGroups(),
            'Did not see tag group ' . $tagGroupId . ' updated'
        );

        return $this;
    }
}
